var Immutable, React, classnames, diff, div, input, keycode, locale, pre, span, tabs;

diff = require('immutablediff');

React = require('react/addons');

keycode = require('keycode');

Immutable = require("immutable");

classnames = require("classnames");

locale = require('./locale');

div = React.createFactory("div");

pre = React.createFactory("pre");

input = React.createFactory("input");

span = document.createElement('span');

tabs = ['action', 'store', 'prev', 'diff'];

module.exports = React.createClass({
  displayName: "actions-recorder-controller",
  mixins: [React.addons.PureRenderMixin],
  propTypes: {
    records: React.PropTypes.instanceOf(Immutable.List),
    inital: React.PropTypes.instanceOf(Immutable.Map),
    updater: React.PropTypes.func.isRequired,
    pointer: React.PropTypes.number.isRequired,
    isTravelling: React.PropTypes.bool.isRequired,
    onCommit: React.PropTypes.func.isRequired,
    onReset: React.PropTypes.func.isRequired,
    onPeek: React.PropTypes.func.isRequired,
    onMergeBefore: React.PropTypes.func.isRequired,
    onClearAfter: React.PropTypes.func.isRequired,
    onRun: React.PropTypes.func.isRequired,
    language: React.PropTypes.string
  },
  getDefaultProps: function() {
    return {
      language: 'en'
    };
  },
  getInitialState: function() {
    return {
      dataPath: '',
      x: 400,
      y: 200,
      tab: 'action'
    };
  },
  onChange: function(event) {
    return this.setState({
      dataPath: event.target.value
    });
  },
  onTabSelect: function(name) {
    return this.setState({
      tab: name
    });
  },
  onKeyDown: function(event) {
    if (event.metaKey && event.ctrlKey) {
      switch (keycode(event.keyCode)) {
        case 'left':
          return this.setState({
            x: this.state.x - 200
          });
        case 'right':
          return this.setState({
            x: this.state.x + 200
          });
        case 'up':
          return this.setState({
            y: this.state.y - 200
          });
        case 'down':
          return this.setState({
            y: this.state.y + 200
          });
      }
    }
  },
  renderItem: function(record, index) {
    var className, onClick;
    onClick = (function(_this) {
      return function() {
        return _this.props.onPeek(index);
      };
    })(this);
    className = classnames("recorder-item", {
      "is-pointer": this.props.isTravelling && this.props.pointer === index
    });
    return div({
      key: index,
      className: className,
      onClick: onClick
    }, record.get(0));
  },
  renderAction: function() {
    var record;
    record = this.props.records.get(this.props.pointer);
    return JSON.stringify(record.get(1), null, 2);
  },
  renderPrev: function() {
    var result, updater;
    updater = (function(_this) {
      return function(acc, record) {
        return _this.props.updater(acc, record.get(0), record.get(1));
      };
    })(this);
    result = this.props.records.slice(0, this.props.pointer).reduce(updater, this.props.initial);
    return this.renderResult(result);
  },
  renderStore: function() {
    var result, updater;
    updater = (function(_this) {
      return function(acc, record) {
        return _this.props.updater(acc, record.get(0), record.get(1));
      };
    })(this);
    result = this.props.records.slice(0, this.props.pointer + 1).reduce(updater, this.props.initial);
    return this.renderResult(result);
  },
  renderDiff: function() {
    var changes, prevResult, result, updater;
    updater = (function(_this) {
      return function(acc, record) {
        return _this.props.updater(acc, record.get(0), record.get(1));
      };
    })(this);
    prevResult = this.props.records.slice(0, this.props.pointer).reduce(updater, this.props.initial);
    result = this.props.records.slice(0, this.props.pointer + 1).reduce(updater, this.props.initial);
    changes = diff(prevResult, result);
    return JSON.stringify(changes, null, 2);
  },
  renderResult: function(result) {
    var dataPath, helper;
    dataPath = this.state.dataPath.split(/\s+/).filter(function(chunk) {
      return chunk.length > 0;
    });
    helper = function(data, keys) {
      var piece;
      if (keys.length === 0) {
        return data;
      } else if ((data != null ? data.get : void 0) != null) {
        piece = data.get(keys[0]);
        if ((piece == null) && (Immutable.List.isList(data))) {
          piece = data.find(function(item) {
            return item.get('id') === keys[0] || item.get('_id') === keys[0];
          });
        }
        return helper(piece, keys.slice(1));
      } else {
        return data;
      }
    };
    return JSON.stringify(helper(result, dataPath), null, 2);
  },
  renderCurrent: function() {
    var store, updater;
    updater = (function(_this) {
      return function(acc, record) {
        return _this.props.updater(acc, record.get(0), record.get(1));
      };
    })(this);
    store = this.props.records.reduce(updater, this.props.initial);
    return div({
      className: 'recorder-current'
    }, pre({
      className: 'recorder-data'
    }, this.renderResult(store)));
  },
  renderDetails: function() {
    return div({
      className: "recorder-details"
    }, div({
      className: 'recorder-mode'
    }, tabs.map((function(_this) {
      return function(name) {
        var onClick, tabClass;
        tabClass = classnames('tap-button', {
          'is-selected': name === _this.state.tab
        });
        onClick = function() {
          return _this.onTabSelect(name);
        };
        return div({
          className: tabClass,
          onClick: onClick,
          key: name
        }, name);
      };
    })(this))), pre({
      className: 'recorder-data'
    }, (function() {
      switch (this.state.tab) {
        case 'action':
          return this.renderAction();
        case 'prev':
          return this.renderPrev();
        case 'store':
          return this.renderStore();
        case 'diff':
          return this.renderDiff();
      }
    }).call(this)));
  },
  render: function() {
    var style;
    style = {
      top: this.state.y,
      left: this.state.x
    };
    return div({
      className: "recorder-controller",
      style: style
    }, div({
      className: "recorder-header"
    }, div({
      className: "tap-button",
      onClick: this.props.onCommit
    }, locale.get('commit', this.props.language)), div({
      className: "tap-button",
      onClick: this.props.onReset
    }, locale.get('reset', this.props.language)), div({
      className: "tap-button",
      onClick: this.props.onMergeBefore
    }, locale.get('mergeBefore', this.props.language)), div({
      className: "tap-button",
      onClick: this.props.onClearAfter
    }, locale.get('clearAfter', this.props.language)), this.props.isTravelling ? div({
      className: "tap-button",
      onClick: this.props.onRun
    }, locale.get('run', this.props.language)) : void 0, input({
      value: this.state.dataPath,
      onChange: this.onChange,
      onKeyDown: this.onKeyDown
    })), div({
      className: 'recorder-viewer'
    }, div({
      className: "recorder-monitor"
    }, this.props.records.map(this.renderItem)), this.props.isTravelling && (this.props.records.get(this.props.pointer) != null) ? this.renderDetails() : this.renderCurrent()));
  }
});
